﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace REG_MARK_LIB_PROJECT{

    public static class REG_CLASS
    {
        private static Dictionary<char, int> letters = new Dictionary<char, int>()
        {
            { 'A', 1 }, { 'B', 2 }, { 'E', 3 },
            { 'K', 4 }, { 'M', 5 }, { 'H', 6 },
            { 'O', 7 }, { 'P', 8 }, { 'C', 9 },
            { 'T', 10 }, { 'Y', 11 }, { 'X', 12 }
        };


        /// <summary>
        /// This method checks contains or not the current region in base of Russian region codes
        /// </summary>
        /// <param name="code">the code of region - int</param>
        /// <returns>true - contains, false - not</returns>
        private static bool CheckRegion(int code)
        {
            var regions = new HashSet<int>();
            var lines = File.ReadAllLines(@"..\\..\\..\\REG_MARK_LIB_PROJECT\\excel\\codes regions.csv").Select(x => x.Split(';'));
   
            foreach (var item in lines)
            {                
                regions.Add(Convert.ToInt32(item[0]));
            }

            return regions.Contains(code);
        }


        public static bool CheckMark(string mark)
        {
            Regex regex = new Regex("^([A-BEKMHOPCTYX]{1})(\\d{3})([A-BEKMHOPCTYX]{2})(\\d{3})$");

            if (regex.IsMatch(mark))
            {
                string code = mark.Substring(mark.Length-3);
                return CheckRegion(Convert.ToInt32(code));
            }

            return false;                       
        }

        public static string GetNextMarkAfter (string mark)
        {
            if (CheckMark(mark))
            {
                string regNumb = mark.Substring(1, 3);

                if (regNumb != "999")
                {
                    regNumb = mark.Substring(0, 6);

                    int nextRegNumb = Convert.ToInt32(regNumb.Substring(1, 3)) + 1;

                    string currentRegNumb = string.Format("{0:d3}", nextRegNumb);

                    mark = mark.Replace(regNumb, regNumb[0] + currentRegNumb + regNumb[4] + regNumb[5]);
                }
                else
                {
                    char[] tillCode = mark.Substring(0, mark.Length - 3).ToCharArray(); // new
                    string chars = mark.Substring(0, mark.Length - 3); // old

                    tillCode[1] = '0';
                    tillCode[2] = '0';
                    tillCode[3] = '0';
                

                    int value = 0;
                    letters.TryGetValue(tillCode[tillCode.Length - 1], out value);

                    if (value != 12)
                    {
                        value++;

                        var myKey = letters.FirstOrDefault(x => x.Value == value).Key;

                        tillCode[tillCode.Length - 1] = myKey;
                    }
                    else
                    {
                        letters.TryGetValue(tillCode[tillCode.Length - 2], out value);
                        if (value != 12)
                        {
                            value++;
                            var myKey = letters.FirstOrDefault(x => x.Value == value).Key;
                            tillCode[tillCode.Length - 2] = myKey;
                        }
                        else
                        {
                            letters.TryGetValue(tillCode[0], out value);
                            if (value != 12)
                            {
                                value++;
                                var myKey = letters.FirstOrDefault(x => x.Value == value).Key;
                                tillCode[0] = myKey;
                                tillCode[tillCode.Length - 1] = 'A';
                                tillCode[tillCode.Length - 2] = 'A';
                            }
                            else
                            {
                                return "error, the max value has been reached in region";
                            }
                        }
                    }
                    string newString = new string(tillCode);
                    mark = mark.Replace(chars, newString);
                }
                return mark;
            }
            else
            {
                return "not existing registration number";
            }
        }

        public static string GetNextMarkAfterInRange(string prevMark, string rangeStart, string rangeEnd)
        {
            string result = "out of stock";

            // the start and end is equals, or the region is incorrect
            if ( rangeStart.Equals(rangeEnd) || rangeStart.Substring(rangeStart.Length-3) != rangeStart.Substring(rangeStart.Length - 3) )
                return result;

            if (string.Compare(rangeStart, prevMark) > 0 || string.Compare(rangeEnd, prevMark) < 0 )
                return result;

            result = GetNextMarkAfter(prevMark);
            return result;
        }

        private static bool CheckRange_InCombination(ref string mark1, ref string mark2)
        {          
            if (CheckMark(mark1) && CheckMark(mark2))
            {
                if (mark2.Substring(mark2.Length-3).Equals(mark1.Substring(mark1.Length-3) )) // regions are equal or not
                {
                    // У нас диапазон, соответственно по условию mark1 < mark2. Т.к. у нас номерные знаки
                    // мы можем вытащить только цифры и буквы, если взять буквы пронумеровать условно
                    // от А, B, E, K ... Х в порядке n+1, то можно устанавливать на основе суммы всех букв и цифр
                    // Какой знак должен быть левым краем диапазона, а какой правым
                    int sumMark2 = Convert.ToInt32(mark2.Substring(1, 3)) + letters[mark2[0]] + letters[mark2[4]] + letters[mark2[5]];
                    int sumMark1 = Convert.ToInt32(mark1.Substring(1, 3)) + letters[mark1[0]] + letters[mark1[4]] + letters[mark1[5]];

                    if (sumMark1 < sumMark2)
                    {
                        return true;
                    }

                    mark1 = "incorrect range of";
                    mark2 = " marks";
                    return false;
                }

                mark1 = "the regions of marks";
                mark2 = " not are same";
                return false;
            }

            mark1 = "incorrect";
            mark2 = " marks or mark";
            return false;
        } 

        public static int GetCombinationsCountInRange(string mark1, string mark2)
        {
            if (CheckRange_InCombination(ref mark1, ref mark2) )
            {
                int count = 1;
                string nextMark = mark1;

                while (!mark2.Equals(nextMark))
                {
                    count++;
                    nextMark = GetNextMarkAfter(nextMark);
                }

                return count;
            }

            throw new Exception(mark1 + mark2);
        }
    }
}
