﻿using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using REG_MARK_LIB_PROJECT;

namespace TestProject
{
    [TestClass]
    public class UnitTest
    {
        //low difficulty level

        // 1
        [TestMethod]
        public void CheckMark_ExistingMark_AndCorrectRegionCode()
        {
            Assert.IsTrue(REG_CLASS.CheckMark("A085BA152"));
        }

        // 2
        [TestMethod]
        public void CheckMark_ExistingMark_IncorrectRegionCode()
        {
            Assert.IsFalse(REG_CLASS.CheckMark("A085BA999"));
        }

        // 3
        [TestMethod]
        public void CheckMark_NotExistingMark_CorrectRegion()
        {
            Assert.IsFalse(REG_CLASS.CheckMark("AF00BA152"));
        }

        // 4
        [TestMethod]
        public void CheckMark_NotExistingMark_IncorrectRegion()
        {
            Assert.IsFalse(REG_CLASS.CheckMark("AF00BA992"));
        }

        // 5
        [TestMethod]
        public void TakeNextMark_ChangeOnlyNumbers()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfter("A000BA152"), "A001BA152");
        }

        // 6
        [TestMethod]
        public void TakeNextMark_ChangeFirstLetter_AndNumbers()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfter("A999BA152"), "A000BB152");
        }

        // 7
        [TestMethod]
        public void ErrorOfMaxValue_OfRegisterNumber_InRegion()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfter("X999XX152"), "error, the max value has been reached in region");
        }

        // 8 Incorrect region code
        [TestMethod]
        public void NextMarkInRange_IncorrectRegion()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfterInRange("X155XB152", "X155XB172", "X155XB152"), "out of stock");
        }

        // 9 There are all values same
        [TestMethod]
        public void NextMarkInRange_SameValues()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfterInRange("X155XB152", "X155XB152", "X155XB152"), "out of stock");
        }

        // 10
        [TestMethod]
        public void NextMarkInRange_TakeNextMark()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfterInRange("X155XB152", "X155XA152", "X155XC152"), "X156XB152");
        }

        //high difficulty level

        /// <summary>
        /// Если бы данный тест не был пройден, функционал программы был бы нарушен
        /// </summary>
        [TestMethod]
        public void TakeNextMark_OfIncorrectMark()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfter("AF00BA152"), "not existing registration number");
        }

        [TestMethod]
        public void TakeNextMark_OfMaxValueOfRegionNumber()
        {
            Assert.AreEqual(REG_CLASS.GetNextMarkAfter("X999XX152"), "error, the max value has been reached in region");
        }

        /// <summary>
        /// Не существующие регионы, а точнее неверные номера
        /// </summary>
        [TestMethod]
        public void CountCombination_OfIncorrectMarkWith_IncorrectRegion()
        {
            var exception = Assert.ThrowsException<Exception>(() => REG_CLASS.GetCombinationsCountInRange("B999BC999", "B999BA999"));
            Assert.AreEqual(exception.Message, "incorrect marks or mark");
        }

        /// <summary>
        /// Различные регионы у номеров
        /// </summary>
        [TestMethod]
        public void CountCombination_NotSameRegions()
        {
            var exception = Assert.ThrowsException<Exception>(() => REG_CLASS.GetCombinationsCountInRange("B999BC152", "B999BA072"));
            Assert.AreEqual(exception.Message, "the regions of marks not are same");
        }

        [TestMethod]
        public void CountCombination_IncorrectRange()
        {
            var exception = Assert.ThrowsException<Exception>(() => REG_CLASS.GetCombinationsCountInRange("B999BC152", "B998BA152"));
            Assert.AreEqual(exception.Message, "incorrect range of marks");
        }

    }
}
